package com.aitrainer.api.test

import com.aitrainer.api.controller.CustomerController
import com.aitrainer.api.model.Customer
import com.aitrainer.api.model.User
import com.aitrainer.api.repository.CustomerRepository
import com.aitrainer.api.service.ServiceBeans
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.HttpHeaders
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import kotlin.test.assertEquals
import kotlin.test.assertNotNull

@SpringBootTest
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class CustomerTests {

    @Autowired
    private var serviceBean: ServiceBeans? = null

    @BeforeAll
    fun init() {
        if ( serviceBean == null ) { serviceBean = ServiceBeans() }
    }

    @Autowired
    private lateinit var customerRepository: CustomerRepository


    private var insertedId: Long? = null

    @Test
    fun testGet() {
        val id: Long = 7
        val customer: Customer = customerRepository.findById( id ).orElse(null)
        assertEquals( customer.name, "Átlag 18 éves fiú")

        val id2: Long = 90
        val controller = CustomerController(customerRepository)
        val response = controller.getCustomerById(id2, HttpHeaders.EMPTY)

        val customer2: Customer = response.body as Customer
        assertNotNull (customer2)

        assertEquals(customer2.email, "sw@andio.biz")
    }

    @Test
    fun testTrainer() {
        val id: Long = 137
        val customers: List<Customer> = customerRepository.findByTrainerId( id )
        assertEquals(customers.count(), 3)
        assertEquals( customers[0].name, "Átlag 13 éves fiú")
    }

    @Test
    fun testInsert() {
        val newCustomer = Customer("Bossanyi", "Tibor", "", 48, "m")
        val savedCustomer: Customer = customerRepository.save(newCustomer)
        assertEquals(savedCustomer.age, 48)

        this.insertedId = savedCustomer.customerId

        val customer: Customer = customerRepository.findById( savedCustomer.customerId ).orElse(null)
        assertEquals( customer.firstname, "Tibor")
        
        this.testUpdate(savedCustomer.customerId)
    }

    fun testUpdate( customerId: Long ) {
        val id: Long = customerId
        assertNotNull(id)

        val updatedCustomer: Customer = customerRepository.findById( id ).orElse(null)

        assertNotNull(updatedCustomer)
        updatedCustomer.firstname ="Tiborka"
        updatedCustomer.dataPolicyAllowed = 1;

        val customer: Customer = customerRepository.save( updatedCustomer )
        assertEquals( customer.firstname, "Tiborka")
        assertEquals(customer.dataPolicyAllowed, 1)

        customerRepository.delete(updatedCustomer)
    }

    @Test
    fun testUpdateCustomer() {
        val id:Long = 90

        val customer: Customer = customerRepository.findById( id ).orElse(null)

        customer.customerId = id
        customer.firstname = "Tib"
        customer.name = "Bossi"
        customer.admin = 1
        customer.active = "Y"
        customer.sex = "m"

        customer.birthYear = 1972
        customer.bodyType = "endomorph"
        customer.dataPolicyAllowed = 1
        customer.fitnessLevel = "advanced"

        val updatedCustomer = customerRepository.save(customer)
        assertEquals(updatedCustomer.customerId, 90)
        assertEquals(updatedCustomer.birthYear, 1972)
        assertEquals(updatedCustomer.fitnessLevel, "advanced")
    }

    @Test
    fun testUpdateCustomer2() {
        var id: Long = 90

        // update no password
        val customer = Customer()

        customer.customerId = id
        customer.password = null
        customer.firstname = "Tib"
        customer.name = "Bossi"
        customer.admin = 1
        customer.sex = "m"
        customer.fitnessLevel = "intermediate"
        //customer.weight = 79
        customer.birthYear = 1972

        val customerController = CustomerController(customerRepository)
        var response: ResponseEntity<*> = customerController.updateCustomerById(id, customer, HttpHeaders.readOnlyHttpHeaders(HttpHeaders.EMPTY) )
        print ("body " + response.body)
        var newCustomer: Customer? = response.body as Customer
        assertEquals(response.statusCode, HttpStatus.OK)
        //assertEquals(newCustomer?.weight, 79)
        assertEquals(newCustomer?.password, "123456789")
        assertEquals(newCustomer?.firstname, "Tib")

        // test not found
        id = 1000
        response = customerController.updateCustomerById(id, customer, HttpHeaders.readOnlyHttpHeaders(HttpHeaders.EMPTY) )
        assertEquals(response.statusCode, HttpStatus.NOT_FOUND)


        // update Password
        id = 103
        customer.password = "blabal"
        customer.firstname = "Kakadu"
        customer.name = "Bos"
        response = customerController.updateCustomerById(id, customer, HttpHeaders.readOnlyHttpHeaders(HttpHeaders.EMPTY) )
        assertEquals(response.statusCode, HttpStatus.OK)
        newCustomer = response.body as Customer
        assertEquals(newCustomer.password, "blabal")
        assertEquals(newCustomer.firstname, "Kakadu")
        assertEquals(newCustomer.name, "Bos")

    }

    @Test
    fun testRegistration() {
        val json = "{\"username\":\"bosi2@example.com\",\"password\":\"94385\",\"firebaseUid\":\"3Firebase8Uid\"}"
        val user: User = User().fromJson(json)
        assertEquals(user.username, "bosi2@example.com")
        val customer = Customer()
        with(customer) {
            email = user.username
            password = user.password
            firebaseUid = user.firebaseUid
        }
        val customerController = CustomerController(customerRepository)
        customerController.serviceBeans = serviceBean
        var response: ResponseEntity<*> = customerController.registration(json)
        print("body " + response.body)
        val newCustomer: Customer = response.body as Customer
        assertEquals(response.statusCode, HttpStatus.OK)
        assertEquals(newCustomer.firebaseUid, "3Firebase8Uid")

        val json2 = "{\"username\":\"bosi2@example.com\",\"password\":\"934345\",\"firebaseUid\":\"3Firebase8Uid\"}"
        response = customerController.registration(json2)
        assertEquals(response.statusCode, HttpStatus.BAD_REQUEST)

        customerRepository.delete(newCustomer)
    }

    @Test
    fun testUpdateFirebaseUid() {
        val newCustomer = Customer("Bossanyi2", "Tibor", "", 48, "m")
        val savedCustomer: Customer = customerRepository.save(newCustomer)
        assertEquals(savedCustomer.age, 48)

        insertedId = savedCustomer.customerId

        val customerController = CustomerController(customerRepository)
        val response: ResponseEntity<*> = customerController.updateCustomerFirebaseUidById(insertedId!!, "3FirebusaeId4")
        val newCustomer2: Customer = response.body as Customer
        assertEquals(response.statusCode, HttpStatus.OK)
        assertEquals(newCustomer2.firebaseUid, "3FirebusaeId4")
        assertEquals(newCustomer2.firstname, "Tibor")
        assertEquals(newCustomer2.name, "Bossanyi2")
        assertEquals(newCustomer2.age, 48)

        customerRepository.delete(newCustomer)
    }

    @Test
    fun testGetCustomerByFirebaseUid() {
        val uid = "3FirebaseU1d"
        val customerController = CustomerController(customerRepository)
        val response: ResponseEntity<*> = customerController.getCustomerByFirebaseUid(uid)
        assertEquals(response.statusCode, HttpStatus.OK)
        val newCustomer: Customer = response.body as Customer
        assertEquals(newCustomer.name, "Bos")
        assertEquals(newCustomer.email, "sw2@andio.biz")

    }

    @Test
    fun testGetCustomerByEmail() {
        val email = "sw2@andio.biz"
        val customerController = CustomerController(customerRepository)
        val response: ResponseEntity<*> = customerController.getCustomerByEmail(email)
        assertEquals(response.statusCode, HttpStatus.OK)

        val newCustomer: Customer = response.body as Customer

        assertEquals(newCustomer.name, "Bos")
        assertEquals(newCustomer.email, "sw2@andio.biz")
        assertEquals(newCustomer.firebaseUid, "3FirebaseU1d")

    }

    /*@Test
    fun testAddCustomerExerciseDevice() {
        val customer: Customer = customerRepository.findById(103) .orElse(null)
        val customerExerciseDevice = CustomerExerciseDevice(
                exerciseDeviceId = 1,
                favourite = false,
                dateAdd = "2020-11-23 20:00"
        )
        customerExerciseDevice.customer = customer
        val listDevice : List<CustomerExerciseDevice> = listOf(customerExerciseDevice)

        customer.exerciseDevices = listDevice
        val newCustomer: Customer = customerRepository.save(customer)

        assertEquals(newCustomer.email, "sw2@andio.biz")
        assertEquals(newCustomer.exerciseDevices.size, 1)
        assertEquals(newCustomer.exerciseDevices[0].exerciseDeviceId, 1)

        val deletedList: List<CustomerExerciseDevice> = listOf()
        customer.exerciseDevices = deletedList
        val newCustomer2 = customerRepository.save(customer)
        assertEquals(newCustomer2.email, "sw2@andio.biz")
        assertEquals(newCustomer2.exerciseDevices.size, 0)

    }
*/
    /*@Test fun _testLogin() {
        val json = "{\"username\":\"bosi2@example.com\",\"password\":\"94333385\"}"
        val user: User = User().fromJson(json)
        val customer = Customer()
        with(customer) {
            email = user.username
            password = user.password
        }
        val customerController = CustomerController(customerRepository)
        customerController.serviceBeans = serviceBean
        var response: ResponseEntity<*> = customerController.registration(json)
        val newCustomer: Customer? = response.body as Customer
        assertEquals(response.statusCode, HttpStatus.OK)

        response = customerController.login(json)
        val loginedCustomer: Customer? = response.body as Customer
        assertEquals(response.statusCode, HttpStatus.OK)
        if ( loginedCustomer != null ) {
            assertEquals(loginedCustomer.email, ("bosi2@example.com") )
        } else {
            assert(true)
        }

        if ( newCustomer != null) {
            customerRepository.delete(newCustomer)
        }
    }*/
}